#include "junction.h"
#include "medium1d.h"
#include <fstream>
#include <iostream>
#include "perf_timer.h"
using namespace std;

static cellSpace* make_model()
{
	int grid_points = (int)ceil(WIDTH/dx);
	cellSpace* space = new cellSpace(cellSpace::SIX_POINT,grid_points);
	for (int x = 0; x < grid_points; x++)
	{
		space->add(new junction(x*dx,Uinit(x*dx)),x);
	}
	return space;
}

static void report(const cellSpace& space, double t)
{
	static ofstream strm("soln");
	static ofstream tstrm("time");
	tstrm << t << endl;
	for (int x = 0; x < space.getWidth(); x++)
	{
		const junction* j = dynamic_cast<const junction*>(space.getModel(x));
		strm << j->pressure() << " ";
//		strm << (j->timeNext()<ADEVS_INFINITY) << " ";
	}
	strm << endl;
}

int main()
{
	cellSpace* space = make_model();
	xdevssim sim(space);
	sim.initialize();
	tic();
	report(*space,0.0);
	while (sim.nextTN() < tend)
	{
		double t = sim.nextTN();
		sim.deltFunc(sim.nextTN());
		report(*space,t);
	}
	cout << toc() << " ms" << endl;
	sim.finalize();
	delete space;
	return 0;
}

