#include <fstream>
#include <iostream>
#include <vector>
#include "medium1d.h"
#include "perf_timer.h"
using namespace std;

#define LEFT 0
#define RIGHT 1
#define CENTER 2

class junction
{
	public:
		junction(double x = 0.0, double UJ0 = 0.0)
		{
			Z[LEFT] = 1.0/(vmax*C(x-0.5*dx));
			Z[RIGHT] = 1.0/(vmax*C(x+0.5*dx));
			Z[CENTER] = vmax*(L(x-0.5*dx)+L(x+0.5*dx))-
			(1.0/C(x-0.5*dx)+1.0/C(x+0.5*dx))/vmax;
			Uout[LEFT] = Uout[RIGHT] = UJ = UJ0;
			delay = 0.0;
		}
		junction(const junction& src)
		{
			(*this) = src;
		}
		const junction& operator=(const junction& src)
		{
			for (int i = 0; i < 3; i++)
			{
				Z[i] = src.Z[i];
			}
			for (int i = 0; i < 2; i++)
			{
				Uout[i] = src.Uout[i];
			}
			delay = src.delay;
			UJ = src.UJ;
			return *this;
		}
		void state_transition(const double* Uin)
		{
			UJ = 2.0*(Z[LEFT]*Uin[LEFT]+Z[RIGHT]*Uin[RIGHT]+Z[CENTER]*delay)/
			(Z[LEFT]+Z[RIGHT]+Z[CENTER]);
			delay = UJ-delay;
			Uout[LEFT] = UJ-Uin[LEFT];
			Uout[RIGHT] = UJ-Uin[RIGHT];
		}
		void output(double* y) const
		{
			y[LEFT] = Uout[LEFT];
			y[RIGHT] = Uout[RIGHT];
		}
		double pressure() const
		{
			return UJ;
		}
		~junction(){}
	private:
		double delay, Uout[2], Z[3], UJ;
};

static vector<junction> make_model()
{
	int grid_points = (int)ceil(WIDTH/dx);
	vector<junction> space;
	for (int x = 0; x < grid_points; x++)
	{
		junction j(x*dx,Uinit(x*dx));
		space.push_back(j);
	}
	return space;
}

static void report(const vector<junction>& space, double t)
{
	static ofstream strm("soln");
	static ofstream tstrm("time");
	tstrm << t << endl;
	vector<junction>::const_iterator i;
	for (i = space.begin(); i != space.end(); i++)
	{
		strm << (*i).pressure() << " ";
	}
	strm << endl;
}

int main()
{
	vector<junction> space(make_model());
	double y[space.size()][2];
	double x[2];
	tic();
	for (int t = 0; t*dx/vmax <= tend; t++)
	{
//		report(space,(double)t*dx/vmax);
		for (size_t j = 0; j < space.size(); j++)
		{
			space[j].output(y[j]);
		}
		for (size_t j = 0; j < space.size(); j++)
		{
			if (j > 0) x[LEFT] = y[j-1][RIGHT];
			else x[LEFT] = 0.0;
			if (j < space.size()-1) x[RIGHT] = y[j+1][LEFT];
			else x[RIGHT] = 0.0;
			space[j].state_transition(x);
		}
	}
	cout << toc() << " ms" << endl;
//	report(space,(double)tend);
	return 0;
}

