#include "junction.h"
#include "medium1d.h"
using namespace std;

junction::junction(double x, double UJ0):
cell_interface(),
atomic()
{
	Z[LEFT] = 1.0/(vmax*C(x-0.5*dx));
	Z[RIGHT] = 1.0/(vmax*C(x+0.5*dx));
	Z[CENTER] = vmax*(L(x-0.5*dx)+L(x+0.5*dx))-
	(1.0/C(x-0.5*dx)+1.0/C(x+0.5*dx))/vmax;
	Uout[LEFT] = Uout[RIGHT] = UJ = UJ0;
	delay = 0.0;
}

void junction::init()
{
	if (UJ != 0.0)
	{
		hold(dx/vmax);
	}
	else
	{
		passivate();
	}
}

void junction::delta_int()
{
	if (Z[CENTER] != 0.0)
	{
		UJ = 2.0*Z[CENTER]*delay/(Z[LEFT]+Z[RIGHT]+Z[CENTER]);
		delay = UJ-delay;
		hold(dx/vmax);
	}
	else
	{
		UJ = delay = 0.0;
		passivate();
	}
	Uout[LEFT] = UJ;
	Uout[RIGHT] = UJ;
}

void junction::delta_ext(double e, const adevs_bag<PortValue>& x)
{
	double Uin[2] = {0.0,0.0};
	for (int i = 0; i < x.getSize(); i++)
	{
		if (x.get(i).value.direction == LEFT)
		{
			Uin[RIGHT] = x.get(i).value.U;
		}
		else
		{
			Uin[LEFT] = x.get(i).value.U;
		}
	}
	UJ = 2.0*(Z[LEFT]*Uin[LEFT]+Z[RIGHT]*Uin[RIGHT]+Z[CENTER]*delay)/
	(Z[LEFT]+Z[RIGHT]+Z[CENTER]);
	delay = UJ-delay;
	Uout[LEFT] = UJ-Uin[LEFT];
	Uout[RIGHT] = UJ-Uin[RIGHT];
	hold(dx/vmax);
}

void junction::delta_conf(const adevs_bag<PortValue>& x)
{
	delta_ext(0.0,x);
}

void junction::output_func(adevs_bag<PortValue>& y)
{
	wave_packet w;
	if (fabs(Uout[LEFT]) > ADEVS_EPSILON) 
	{
		w.direction = LEFT;
		w.U = Uout[LEFT];
		output(cell_interface::dout[0][1][1],w,y);
	}
	if (fabs(Uout[RIGHT]) > ADEVS_EPSILON) 
	{
		w.direction = RIGHT;
		w.U = Uout[RIGHT];
		output(cell_interface::dout[2][1][1],w,y);
	}
}

void junction::save_state()
{
	state_t s;
	s.tL = timeLast();
	s.tN = timeNext();
	s.UJ = UJ;
	s.delay = delay;
	s.Uout[LEFT] = Uout[LEFT];
	s.Uout[RIGHT] = Uout[RIGHT];
	iss.save_state(s,timeLast());
}

void junction::restore_state(double t)
{
	state_t s(iss.restore_state(t));
	setTimeNext(s.tN);
	setTimeLast(s.tL);
	UJ = s.UJ;
	delay = s.delay;
	Uout[LEFT] = s.Uout[LEFT];
	Uout[RIGHT] = s.Uout[RIGHT];
}

void junction::remove_fossils(double t_old)
{
	iss.remove_fossils(t_old);
}
