#include "Glendale.h"

const double Glendale::K_s = 1.5162;
const double Glendale::theta_s = 0.4686;
const double Glendale::theta_r = 0.1060;
const double Glendale::alpha = 1.04;
const double Glendale::n = 1.15162;

double Glendale::width() const
{
	return 1.5;
}

double Glendale::height() const
{
	return 0.16;
}

double Glendale::h0(double x, double y) const
{
	return -10.0;
}

double Glendale::qs(double x, double y) const
{
/*
	if (y > 0.13 && x == 0.0)
	{
		return 1.62;
	}
	else
	{
		return 0.0;
	}
*/
	if (sqrt((y-height()/2.)*(y-height()/2.)+(x-width()/2.)*(x-width()/2.)) < 0.1)
	{
		return 1.62;
	}
	else
	{
		return 0.0;
	}
}

double Glendale::K(double x, double y, double h) const
{
	double m = 1.0-1.0/n;
	if (!saturated(x,y,h))
	{
		double ah_1 = pow(fabs(alpha*h),n-1.0);
		double ah = pow(fabs(alpha*h),n);
		return K_s * 
		pow((1.0-ah_1)*
		pow(1.0+ah,-m),2.0) /
		pow(1.0+ah,m/2.0);
	}
	else
	{
		return K_s;
	}
}

double Glendale::theta(double x, double y, double h) const
{
	double m = 1.0-1.0/n;
	if (!saturated(x,y,h))
	{
		return (theta_s-theta_r)*
		pow(fabs(1.0+pow(fabs(alpha*h),n)),-m)+theta_r;
	}
	else
	{
		return theta_s;
	}
}

double Glendale::C_Ss(double x, double y, double h) const
{
	double m = 1.0-1.0/n;
	if (!saturated(x,y,h))
	{
		return alpha*m*n*(theta_s-theta_r) *
		pow(fabs(alpha*h),n-1.0) / 
		pow(1.0+pow(fabs(alpha*h),n),m+1.0);
	}
	else
	{
		return alpha+n;
	}
}

bool Glendale::saturated(double x, double y, double h) const
{
	return h >= -ADEVS_EPSILON;
}
