#include "cell.h"
using namespace std;

#define UP 0
#define DOWN 1
#define LEFT 2
#define RIGHT 3
#define CENTER 4

double cell::ds = 0.001;
double cell::D = 0.1;

cell::cell(const MaterialParams* p, double x, double y):
atomic(),
cell_interface(),
x(x),
y(y),
p(p)
{
}

cell::cell(const cell& src):
atomic(src),
cell_interface(src),
x(src.x),
y(src.y),
h(src.h),
p(src.p)
{
	for (int i = 0; i < 4; i++) 
	{
		hn[i] = src.hn[i];
	}
}

void cell::init()
{
	hn[LEFT] = p->h0(x-ds,y);
	hn[RIGHT] = p->h0(x+ds,y);
	hn[UP] = p->h0(x,y-ds);
	hn[DOWN] = p->h0(x,y+ds);
	K[LEFT] = p->K(x-ds,y,hn[LEFT]);
	K[RIGHT] = p->K(x+ds,y,hn[RIGHT]);
	K[UP] = p->K(x,y-ds,hn[UP]);
	K[DOWN] = p->K(x,y+ds,hn[DOWN]);
	h.setParams(D);
	double p0 = p->h0(x,y);
	K[CENTER] = p->K(x,y,p0);
	C_Ss = p->C_Ss(x,y,p0);
	double dh0 = dh(p->h0(x,y));
	h.init(&p0,&dh0);
	hold(h.ta());
}

void cell::delta_int()
{
	K[CENTER] = p->K(x,y,h.out());
	C_Ss = p->C_Ss(x,y,h.out());
	double dhn = dh(h.out());
	h.integ(h.ta(),&dhn);
	hold(h.ta());
}

void cell::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	scan_input(x);
	double dhn = dh(h.qstate());
	h.integ(e,&dhn);
	hold(h.ta());
}

void cell::delta_conf(const adevs_bag<PortValue>& x)
{
	scan_input(x);
	delta_int();
}

void cell::output_func(adevs_bag<PortValue>& y)
{
	output(cell_interface::out,h.out(),y);
}

void cell::scan_input(const adevs_bag<PortValue>& xb)
{
	for (int i = 0; i < xb.getSize(); i++)
	{
		if (xb.get(i).port == cell_interface::in[0][1][1]) 
		{		
			hn[LEFT] = xb.get(i).value;
			K[LEFT] = p->K(x-ds,y,hn[LEFT]);
		}
		else if (xb.get(i).port == cell_interface::in[2][1][1]) 
		{
			hn[RIGHT] = xb.get(i).value;
			K[RIGHT] = p->K(x+ds,y,hn[RIGHT]);
		}
		else if (xb.get(i).port == cell_interface::in[1][0][1]) 
		{
			hn[UP] = xb.get(i).value;
			K[UP] = p->K(x,y-ds,hn[UP]);
		}
		else if (xb.get(i).port == cell_interface::in[1][2][1]) 
		{
			hn[DOWN] = xb.get(i).value;
			K[DOWN] = p->K(x,y+ds,hn[DOWN]);
		}
	}
}

double cell::getState() const
{
	return h.qstate();
}

void cell::bc(double hd) 
{
	if (x <= 0.0) hn[LEFT] = hd;
	else if (x >= p->width()) hn[RIGHT] = hd;
	if (y <= 0.0) hn[UP] = hd;
	else if (y >= p->height()) hn[DOWN] = hd;
}

double cell::dh(double hd)
{
	bc(hd);
	double Kip_x = (K[RIGHT]+K[CENTER])/2.0;
	double Kim_x = (K[LEFT]+K[CENTER])/2.0;
	double Kip_y = (K[DOWN]+K[CENTER])/2.0;
	double Kim_y = (K[UP]+K[CENTER])/2.0;
	double dd =  ((Kip_x*(hn[RIGHT]-hd)-Kim_x*(hd-hn[LEFT])+
	Kip_y*(hn[DOWN]-hd)-Kim_y*(hd-hn[UP]))/(ds*ds) + p->qs(x,y))/C_Ss;
	return dd;
}
