#include "cell.h"
#include "Glendale.h"
#include <iostream>
#include <fstream>
using namespace std;

/**
Create the sandbox model.
*/
static MaterialParams* p = NULL;
static cellSpace* make_sandbox()
{
	// Set the cell size to 0.005 m
	cell::ds = 0.005;
	// Set the integration quantum to the sqrt of the cell size
	cell::D = 0.01;
	// Get the sand material parameters
	p = new Glendale();
	// Create the cellspace for a sandbox 1.5 m x 0.16 m
	cellSpace* sandbox = 
		new cellSpace(cellSpace::SIX_POINT,
			(int)ceil(p->width()/cell::ds),(int)ceil(p->height()/cell::ds));
	for (int i = 0; i < sandbox->getWidth(); i++)
	{
		for (int j = 0; j < sandbox->getHeight(); j++)
		{
			double x = i * cell::ds;
			double y = j * cell::ds;
			sandbox->add(new cell(p,x,y),i,j);
		}
	}
	// Done
	return sandbox;
}

/**
Destroy the sandbox model.
*/
static void delete_sandbox(cellSpace* sandbox)
{
	delete sandbox;
	delete p;
}

/**
Print a report of the sandbox state.
*/
static void report(cellSpace* sandbox)
{
	ofstream strm ("soln");
	for (int j = 0; j < sandbox->getHeight(); j++)
	{
		for (int i = 0; i < sandbox->getWidth(); i++)
		{
			const cell* c = dynamic_cast<const cell*>(sandbox->getModel(i,j));
			strm << c->getState() << " ";
		}
		strm << endl;
	}
	strm.close();
}

int main()
{
	double tend = 0.1;
	cellSpace* sandbox = make_sandbox();
	xdevssim sim(sandbox);
	sim.initialize();
	int count = 0;
	while (sim.nextTN() < tend)
	{
		if (count == 0)
		{
			printf("\r%3.2f %%\t",sim.nextTN()/tend*100.0);
		}
		count = (count+1)%2000;
		sim.deltFunc(sim.nextTN());
	}
	printf("\n");
	report(sandbox);
	sim.finalize();
	delete_sandbox(sandbox);
	return 0;
}
