#include <iostream>
#include "adevs.h"
using namespace std;

/**
Solution to dy/dt = -y with y(0) = 0.  Solution is calculated using
the QSS1 integrator.
*/

// Differential function
static double f(double y)
{
	return -y;
}

int main()
{
	// Initial condition
	double y = 1.0;
	// Initial value of differential function
	double dy = f(y);
	// Simulation clock
	double t = 0.0;
	// Integration quantum
	double D = 0.11;
	// Create and initialize the QSS1 integrator
	qode q;
	q.setParams(D);
	q.init(y,dy);
	// Integrator from 0 to 30 or until the model passivates
	// (i.e., reaches steady state)
	while (t < 30.0 && q.ta() < ADEVS_INFINITY)
	{
		// Print current value
		cout << t << " " << y << endl;
		// Compute the next event time
		t += q.ta();
		// Get value of the integral at that time (i.e., the integrator output)
		y = q.out();
		// Get new value of differential function
		dy = f(y);
		// Perform the next integration step
		q.integ(q.ta(),dy);
	}
	// !Done
	return 0;
}
